/**
 * Copyright 2020 IBM Corp. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package org.opengroup.osdu.register.provider.ibm.action.datastore;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.tenant.TenantInfo;
import org.opengroup.osdu.core.common.util.Crc32c;
import org.opengroup.osdu.register.action.model.Action;
import org.opengroup.osdu.register.provider.ibm.ddms.datastore.DatastoreMultiTenantAccess;
import org.opengroup.osdu.register.provider.interfaces.action.IActionRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.cloudant.client.api.Database;
import com.cloudant.client.org.lightcouch.DocumentConflictException;
import com.google.gson.Gson;

import lombok.Getter;

@Repository
public class ActionRepository implements IActionRepo {

    private static final String ACTION_DB_NAME = "action";

    @Autowired
    private JaxRsDpsLog log;
    @Autowired
    private DatastoreMultiTenantAccess dataStoreTenants;
    @Autowired
    private TenantInfo tenantInfo;

    @Override
    public Action createAction(Action a) {
        return createActionInDatastore(a);
    }

    @Override
    public Action get(String id) {
        Database db = dataStoreTenants.get(getDbNameWithTenant());
        if(!(db.contains(id))) {
        	log.error(String.format("Action with id %s does not exist.", id));
        	throw new AppException(404, "Not found", String.format("Action with id %s does not exist.", id));
        }
        ActionDoc actionDoc = db.find(ActionDoc.class, id);
        return actionDoc.getAction();
    }

    @Override
    public boolean delete(String id) {
        return deleteFromDatastore(id);
    }

    private Action createActionInDatastore(Action action) {
        Database db = dataStoreTenants.get(getDbNameWithTenant());
        ActionDoc actionDoc = new ActionDoc(action);
        try {
			db.save(actionDoc);
		} catch (DocumentConflictException e) {
			throwConflict();
		} 
        return action;
    }

    private Action throwConflict() {
        throw new AppException(409, "Conflict", "An action already exists with the same message and endpoint combination");
    }

    private boolean deleteFromDatastore(String id) {
        Database db = dataStoreTenants.get(getDbNameWithTenant());
        boolean output = false;
        if(db.contains(id)) {
        	ActionDoc actionDoc = db.find(ActionDoc.class, id);
            db.remove(actionDoc);
            output = true;
        }
        else {
        	log.error(String.format("deletion failed. Could not find action id : %s", id));
        	output=false;
        }
        return output;

    }

    @Override
    public List<Action> getAllActions() {
        Database db = dataStoreTenants.get(getDbNameWithTenant());
        List<Action> actionsList=new ArrayList<>();
		try {
			List<ActionDoc> actionDocsList = db.getAllDocsRequestBuilder().includeDocs(true).build().getResponse().getDocsAs(ActionDoc.class);
			actionsList = actionDocsList.stream().map(i->i.getAction()).collect(Collectors.toList());
		} catch (IOException e) {
			log.error("IOException while fetching all actons",e);
			e.printStackTrace();
		}
        return actionsList;
    }

    
	/**
	 * @return
	 */
	private String getDbNameWithTenant() {
		return tenantInfo.getName()+"-"+ACTION_DB_NAME;
	}

}
